<?php

/**
 * Implements hook_drush_help().
 */
function commerce_fulfilment_oms_drush_help($section) {
  switch ($section) {
    case 'drush:check-oms-status':
      return dt('Rebuild the registry or module cache in a Drupal install.');
  }
}


/**
 * Implements hook_drush_command().
 */
function commerce_fulfilment_oms_drush_command() {
  $items = array();

  $items['check-oms-status'] = array(
    'description' => 'Check order status against OMS.',
    'callback' => 'commerce_fulfilment_oms_drush_checkstatus',
    'options' => array(
      'dry-run' => 'Do no fire the rules event.',
    ),
    'examples' => array(
      'drush cos --dry-run' => 'Check the order status and report the results, but do not fire rules event.',
    ),
    'aliases' => array('cos'),
  );

  return $items;
}

/**
 * Check all "processing" orders against OMS.
 *
 * If the order is completed, fire an event to let Rules take care of it.
 */
function commerce_fulfilment_oms_drush_checkstatus() {
  $orders = commerce_order_load_multiple(array(), array('status' => 'processing'));
  if (count($orders) === 0) {
    drush_log(dt('No orders found with status "processing".'), 'success');
    return;
  }

  $order_ids = array_keys($orders);

  // Attempt to export the orders or fail with an error
  try {
    $client = new CommerceOMSOrderTaker;
    $parameters = new ExportOrders('SHP', $order_ids);
    $response = $client->ExportOrders($parameters);
  } catch (Exception $e) {
    $vars = array('%error' => $e->getMessage());
    watchdog('commerce_fulfilment_oms', 'Failed to fetch order status from OMS: %error', $vars, WATCHDOG_ERROR);
    drush_log(dt('Unable to fetch order status from OMS.'), 'warning');
    return;
  }

  // Check that we have a response we understand
  if (isset($response->ExportOrdersResult->Orders->Order)) {
    $obj = $response->ExportOrdersResult->Orders->Order;
    if (is_array($obj)) {
      $oms_orders = $obj;
    } else {
      $oms_orders = array($obj);
    }
  } else {
    drush_log(dt('Unable to parse order status response.', array('@moid' => $order_id)), 'warning');
    return;
  }

  // Make a lookup table of the orders vs OMS orders
  //
  // OMS could have multiple orders for the same merchant ID (e.g. if it's resent)
  // so we only use the latest order.
  $order_lookuptable = array();
  foreach ($oms_orders as $oms_order) {
    if (isset($order_lookuptable[$oms_order->Merchant_ID])) {
      $replacement = strtotime($oms_order->DateCreated);
      $current = strtotime($order_lookuptable[$oms_order->Merchant_ID]->DateCreated);
      if ($current > $replacement) {
        echo sprintf("no good %s %s\n", $oms_order->DateCreated, $oms_order->Order_Number);
        continue;
      }
    }
    $order_lookuptable[$oms_order->Merchant_ID] = $oms_order;
  }

  // Iterate the orders and fire the event.
  $completed = $failed = 0;
  foreach ($orders as $order) {
    if (!isset($order_lookuptable[$order->order_id])) {
      $failed++;
      drush_log(dt('Could not find order @order_id in OMS export', array('@order_id' => $order->id)));
    }
    $oms_order = $order_lookuptable[$order->order_id];

    if ($oms_order->Status === 'Complete') {
      $shipment = $oms_order->Shipments->Shipment;
      $tracking_url = commerce_fulfilment_oms_get_tracking_url($shipment->ShipMethod, $shipment->TrackingNumber);
      $order->tracking_url = $tracking_url; // Wasn't able to find a better way to do this.

      if (!drush_get_option('dry-run')) {
        rules_invoke_event('commerce_fulfilment_oms_rules_orderfulfiled_event', $order);
      }
      $completed++;
    }
  }

  $vars = array('@total' => count($orders), '@completed' => $completed, '@failed' => $failed);
  drush_log(dt('Reviewed @total orders, @failed failed, reported @completed completed orders.', $vars), 'success');
}
