<?php

/**
 * @file This file maps the SOAP datatypes to PHP classes.
 */


/**
 * Our custom SOAP class to ensure the XML is in the same format that OMS requires.
 */
class CommerceOMSSoapClient extends SoapClient
{
  public function __doRequest($request, $location, $action, $version) {
    preg_match('|xmlns:ns1="(.*)"|', $request, $matches);
    $namespace = $matches[1];

    $request = str_replace('SOAP-ENV', 'soap', $request);
    $request = preg_replace('/<ns1:(\w+)/', '<$1 xmlns="'.$namespace.'"', $request, 1);
    $request = preg_replace('/<ns1:(\w+)/', '<$1', $request);
    $request = str_replace(array('/ns1:', 'xmlns:ns1="'.$namespace.'"'), array('/', ''), $request);

    return parent::__doRequest($request, $location, $action, $version);
  }
}


/**
 * Our custom SOAP class to ensure the XML is in the same format that OMS requires.
 */
class CommerceOMSOrderTaker extends CommerceOMSSoapClient
{
  public function __construct() {
    $wsdl_url = variable_get('commerce_fulfilment_oms_ordertaker_wsdl', '');
    parent::__construct($wsdl_url);
  }
}


/**
 * Our custom SOAP class to ensure the XML is in the same format that OMS requires.
 */
class CommerceOMSInventoryManager extends CommerceOMSSoapClient
{
  public function __construct() {
    $wsdl_url = variable_get('commerce_fulfilment_oms_inventorymanager_wsdl', '');
    parent::__construct($wsdl_url);
  }
}


/**
 * ViewStockLevels
 */
class ViewStockLevels {
  public $strB2BCustomerName;
  public $strB2BCustomerEmail;
  public $strB2BCustomerPassword;

  public $strSKU;
  public $dtmStartDate;
  public $strTimeZone;

  function __construct($sku) {
    $this->strB2BCustomerName = variable_get('commerce_fulfilment_oms_customer_name', '');
    $this->strB2BCustomerEmail = variable_get('commerce_fulfilment_oms_customer_email', '');
    $this->strB2BCustomerPassword = variable_get('commerce_fulfilment_oms_customer_password', '');

    $this->strSKU = new stdClass;
    $this->strSKU->string = $sku;
  }
}


/**
 * ExportOrders
 */
class ExportOrders {
  public $strB2BCustomerName;
  public $strB2BCustomerEmail;
  public $strB2BCustomerPassword;

  /**
   * @var Transaction type
   * SHP = Sales Order
   * RMA = Return Merchandise Authorization. This is a return document that can be changed.
   * RO = ?
   * PO = Purchase Order. This is a Purchase Order Document that is in its original version and has not been changed.
   * ASN = Advanced Shipment Notice. This is a version of the Purchase Order that can be changed.
   * CXL = Canceled/Voided order;
   */
  public $strTransactionType;

  /**
   * Multiple merchant orders can be passed to the method. To pass in multiple
   * merchant orders make sure the order numbers are separated a comma.
   * Example: “10001,10002”
   */
  public $strOrderID;
  public $strMerchantOrderID;

  /**
   * Specify this value only to return orders in a specific status. If you are
   * going to enter multiple statuses, each status needs to be separated a comma.
   *
   * For example, if you want all completed and voided orders then enter "Complete, Void".
   * If this parameter is left blank it will return orders in all statuses.
   */
  public $strOrderStatus;

  /**
   * If a date time is provided, the method will return only those orders that
   * have been updated and created since the date and time provided.
   */
  public $dtmOrdersFromDate;

  public $strTimeZone;
  public $strClientRoleName = 'Web Customers';

  /**
   * Construct the ExportOrders
   *
   * @var $type array The type of ExportOrders request to send (TransactionType)
   * @var $order_id string|array The order_id or order_ids to send.
   */
  function __construct($type, $order_id) {
    $this->strB2BCustomerName = variable_get('commerce_fulfilment_oms_customer_name', '');
    $this->strB2BCustomerEmail = variable_get('commerce_fulfilment_oms_customer_email', '');
    $this->strB2BCustomerPassword = variable_get('commerce_fulfilment_oms_customer_password', '');

    if (is_array($order_id)) {
      $order_id = implode(',', $order_id);
    }

    $this->strTransactionType = $type;
    $this->strMerchantOrderID = $order_id;
  }
}


/**
 * CreateSalesOrder
 *
 * @see https://ws.ordermanagementsystems.com.au/omsordertaker/omsordertaker.asmx?op=CreateSalesOrder
 */
class CreateSalesOrder {

  public $LiveMode;
  public $order;

  /**
   * Construct a CreateSalesOrder object with the structure required by the webservice.
   *
   * @var $credentials array The B2B login credentials.
   * @var $oe EntityDrupalWrapper The order to process.
   * @var $shipping_method string The shipping method.
   * @var $live_mode bool The value for LiveMode on the CreateSalesOrder object.
   */
  function __construct(EntityDrupalWrapper $oe, $live_mode = TRUE) {

    $order = new SalesOrder;

    $order->B2BCustomerName = variable_get('commerce_fulfilment_oms_customer_name', '');
    $order->B2BCustomerEmail = variable_get('commerce_fulfilment_oms_customer_email', '');
    $order->B2BCustomerPassword = variable_get('commerce_fulfilment_oms_customer_password', '');

    $order->Status = 'Processing';

    $order->EmailAddress = $oe->mail->value();
    $order->ExternalID = $oe->order_number->value();
    $order->ClientRoleName = 'Web Customers';

    // Create the OrderLineItem objects.
    $lineitems = array();
    foreach ($oe->commerce_line_items as $i => $item) {
      if ($item->type->value() !== 'product') {
        continue;
      }
      $price = $item->commerce_unit_price->value();

      $li = new OrderLineItem;
      $li->LineNumber = $i;
      $li->ItemName = $item->line_item_label->value();
      $li->QuantityOrdered = $item->quantity->value();
      $li->SKU = $item->line_item_label->value();
      $li->UnitPrice = $price['amount'] / 100;
      $li->OrderLineItem3PLID = $item->line_item_label->value();
      $lineitems[] = $li;
    }
    $order->OrderLineItems = $lineitems;

    // Bill to
    $billaddr = $oe->commerce_customer_billing->commerce_customer_address;
    list($firstname, $lastname) = commerce_fulfilment_oms_split_name(trim($billaddr->name_line->value()));
    $order->BillToFName = $firstname;
    $order->BillToLName = $lastname;
    $order->BillToAddress1 = $billaddr->thoroughfare->value();
    $order->BillToAddress2 = $billaddr->premise->value();
    $order->BillToCity = $billaddr->locality->value();
    $order->BillToState = $billaddr->administrative_area->value();
    $order->BillToZip = $billaddr->postal_code->value();
    $order->BillToCountry = $billaddr->country->value();

    // Ship to
    $shipaddr = $oe->commerce_customer_shipping->commerce_customer_address;
    list($firstname, $lastname) = commerce_fulfilment_oms_split_name(trim($shipaddr->name_line->value()));
    $order->ShipToSameAsBillTo = FALSE;
    $order->ShipToEmailAddress = $oe->mail->value();
    $order->ShipToFName = $firstname;
    $order->ShipToLName = $lastname;
    $order->ShipToAddress1 = $shipaddr->thoroughfare->value();
    $order->ShipToAddress2 = $shipaddr->premise->value();
    $order->ShipToCity = $shipaddr->locality->value();
    $order->ShipToState = $shipaddr->administrative_area->value();
    $order->ShipToZip = $shipaddr->postal_code->value();
    $order->ShipToCountry = $shipaddr->country->value();

    // Other
    $order->OrderDate = date(DATE_W3C, $oe->created->value());
    $order->MerchantOrderNumber = $oe->order_number->value();
    $order->Comments = $oe->field_order_notes->value();

    // Ship method
    $shipping_lineitem = commerce_fulfilment_get_shipping_lineitem($oe);
    if (!$shipping_lineitem) {
      $shipmethod = variable_get('commerce_fulfilment_oms_default_shipping', '');
    } else {
      $shipmethod = $shipping_lineitem->value()->data['shipping_service']['title'];
    }
    $order->ShipMethod = $shipmethod;

    // Build the CreateSalesOrder wrapper.
    $this->order = $order;
    $this->LiveMode = $live_mode;
  }
}


/**
 * SalesOrder
 *
 * This is a complex type defined by CreateSalesOrder. This class lists all the properties as required by
 * the SOAP WSDL, and sets some sensible defaults and data types.
 *
 * @see https://ws.ordermanagementsystems.com.au/omsordertaker/omsordertaker.asmx?op=CreateSalesOrder
 */
class SalesOrder {
  function __construct() {
    $this->RequestedShipDate = date(DATE_W3C);
    $this->RequestedDeliveryDate = date(DATE_W3C);
  }

  public $TransCode;
  public $AuthCode;
  public $Status = 'Processing';
  public $Origin = 'Other'; // set to Drupal eventually

  public $EmailAddress;
  public $ExternalID;
  public $ClientRoleName = 'Web Customers';
  public $RefID = 0;

  public $B2BCustomerName;
  public $B2BCustomerEmail;
  public $B2BCustomerPassword;


  // Bill to
  public $BillToCompany;
  public $BillToFName;
  public $BillToLName;
  public $BillToAddress1;
  public $BillToAddress2;
  public $BillToCity;
  public $BillToState;
  public $BillToZip;
  public $BillToCountry;
  public $BillToPhone;
  public $BillToResidential = TRUE;
  public $ShipToSameAsBillTo;

  // Shop to
  public $ShipToCompany;
  public $ShipToFName;
  public $ShipToLName;
  public $ShipToAddress1;
  public $ShipToAddress2;
  public $ShipToCity;
  public $ShipToState;
  public $ShipToZip;
  public $ShipToCountry;
  public $ShipToPhone;
  public $ShipToResidential = TRUE;

  // Other
  public $OrderDate;
  public $RequestedShipDate;
  public $RequestedDeliveryDate;
  public $MerchantPONumber;
  public $MerchantOrderNumber;
  public $Comments;
  public $PricingDiscount = 0.00;
  public $CouponDiscount = 0.00;
  public $Handling = 0.00;
  public $FlatRateShipping = 0.00;
  public $Tax = 0.00;
  public $ShipFee = 0.00;
  public $ShipMethod;
  public $ShipAccount;
  public $POTerm = 'Net-10';
  public $Priority = 'Low';
  public $ReferrerName;
  public $PaymentType;
  public $NameOnCard;
  public $CreditCardNumber;
  public $CreditCardExpMon = 0;
  public $CreditCardExpYr = 0;
  public $OptIn = FALSE;
  public $EmailCustomer = FALSE;
  public $BillToShipTo3PLID;
  public $Currency;
  public $Language;
  public $CheckForDuplicates = TRUE;
  public $HoursForDuplicateCheck = 48;
  public $OrderLineItems;
  public $OrderPayments;
  public $OrderComments;
  public $ValidateOnlyMOrderID = TRUE;
  public $LoginClientUserRoleID = 0;
  public $IPAddress;
  public $ShipToEmailAddress;
  public $NIPNumber;
  public $CheckOverrideLineItemTotal = FALSE;
  public $OverrideLineItemTotalAmount = 0;

  // EDI
  public $EDI_TransactionType;
  public $EDI_EmailClient = TRUE;
  public $EDI_FileName;
  public $EDI_NotBefore = 0;
  public $EDI_CancelAfter = 0;
  public $EDI_Description1;
  public $EDI_Description2;
  public $EDI_Description3;
  public $EDI_Description4;
  public $EDI_Description5;
}


/**
 * OrderLineItem
 *
 * This is a complex type defined by CreateSalesOrder representing the lineitems
 * for this order.
 *
 * This class lists all the properties as required by
 * the SOAP WSDL, and sets some sensible defaults and data types.
 *
 * @see https://ws.ordermanagementsystems.com.au/omsordertaker/omsordertaker.asmx?op=CreateSalesOrder
 */
class OrderLineItem {
  public $LineNumber;
  public $ItemName;
  public $QuantityOrdered;
  public $SKU;
  public $SKUWMS;
  public $UnitPrice;
  public $UnitPriceDiscount = 0;
  public $WarehouseName;
  public $OrderLineItem3PLID;
  public $ItemBarCode;
  public $Warehouse3PLID;
  public $SubInventory3PLID;
  public $BuyLoc;
  public $ShipLoc;
}
